%% find the SG solution and Elo solutions

log10e=log10(exp(1));
ee=1.e-5; %gradient precision

% Gaussian
PHI_G = @(x,sig) qfunc(-x/sig);
PHI_G_prim = @(x,sig) 1/sig* exp( -(x/sig).^2/2 )/sqrt(2*pi);
PHI_G_bis = @(x,sig)  -(x/sig^3).*exp(-(x/sig).^2/2)/sqrt(2*pi);
xi_G = @(x,sig)  PHI_G_prim(x,sig)./ ( PHI_G(x,sig).*PHI_G(-x,sig) );
% Logistic
PHI_L = @(x,sig)  1./(1+10.^(-x./sig));
PHI_L_prim = @(x,sig) 1/(sig*log10e) * 10.^(-x./sig)./(1+10.^(-x./sig)).^2;
PHI_L_bis = @(x,sig) 1/(sig*log10e)^2 * 10.^(-x./sig).*(10.^(-x./sig)-1)./(1+10.^(-x./sig)).^3;
xi_L = @(x,sig)  PHI_L_prim(x,sig)./ ( PHI_L(x,sig).* PHI_L(-x,sig) );

% Laplace
PHI_Lap = @(x,sig)  0.5*exp(x/sig).*(x<0) + (1-0.5*exp(-x/sig)).*(x>=0);
%PHI_prim = @(x) 0.5*exp(-abs(x));
%PHI_bis = @(x) -0.5*sign(x).*exp(-abs(x));


DATA_MODEL='Gauss', %'Logistic';'Gauss', 'Laplace'

sig_model=1;

%theta_0 =[0 0.5 2 2.2]';% Gauss
theta_0 =[0 0.2 0.5 5 5.2 5.5]';% Logistic

M=length(theta_0);

% x12= [1 -1 0]';
% x13= [1 0 -1]';
% x23= [0 1 -1]';
% X = [x12 x13 x23];

%%% find all possible matches
X=[];
for m=1:M-1
    xx= [zeros(m-1,M-m); ones(1,M-m); -eye(M-m)];
    X=[X, xx ];
end
v0 = X'*theta_0; % all possible differences

switch DATA_MODEL
    case 'Gauss'
        p_ij= PHI_G(v0, sig_model);  %%% Gaussian data
        C = PHI_G( theta_0-theta_0', sig_model );
    case 'Logistic'
        p_ij= PHI_L(v0, sig_model);  %%% logistic data
        C = PHI_L( theta_0-theta_0', sig_model );
    case 'Laplace'
        p_ij= PHI_Lap(v0, sig_model);  %%% Laplace data
        C = PHI_Lap( theta_0-theta_0', sig_model );
end
%p_ij'
J_opt = -(  p_ij'*log(p_ij) + (1-p_ij)'*log(1-p_ij) )/M


It_max=1.e+4;

%%% Gaussian fit
if 0
    sig_G = 600;
    theta = 1000+zeros(M,1)*0; %sig_G/sig_model;
    %s= theta;
    mu=300;
    it=1;
    J_G = zeros(It_max,1);
    
    while 1
        v= X'*theta;
        J_G(it)= -(  p_ij'*log(PHI_G(v, sig_G)) + (1-p_ij)'*log(PHI_G(-v, sig_G)) )/M;
        
        grad = -X*( ( p_ij - PHI_G(v, sig_G) ).*xi_G(v, sig_G) );
        d = grad - (sum(grad))/M;
        theta = theta - mu*d;
        
        %%%%
        it = it+1;
        if it>It_max %|| norm(grad)<ee,
            theta_G = theta;
            v=X'*theta;
            J_G_est= -(  p_ij'*log(PHI_G(v, sig_G)) + (1-p_ij)'*log(PHI_G(-v, sig_G)) )/M
            PHI_G( v, sig_G)'
            break;
        end
    end
    
    %%%% Hessian
    dS= theta - theta';
    PHI_val=PHI_G(dS, sig_G);
    g_fun = PHI_G_prim(dS, sig_G)./PHI_val;
    h_func = PHI_G_bis(dS, sig_G)./PHI_val - g_fun.^2;
    B = C.*h_func;
    H = ( B + B') - diag(sum(B+B',1),0);  %% Hessian
    %%%%%%%
    [Q,LL]=eig(H); diag(LL)'
end

%%% Logistic fit
if 1
    sig_L = 600;
    
    theta = 1000+zeros(M,1)*0; %sig_G/sig_model;
    mu=40;
    it=1;
    J_L = zeros(It_max,1);
    
    while 1
        % new
        v= X'*theta;
        J_L(it)= -(  p_ij'*log(PHI_L(v, sig_L)) + (1-p_ij)'*log(PHI_L(-v, sig_L)) )/M;
        
        grad = -X*( ( p_ij - PHI_L(v, sig_L) ) );
        d = grad - (sum(grad))/M;
        theta = theta - mu*d;
        
        %%%%
        it = it+1;
        if it>It_max %|| norm(grad)<ee
            theta_L = theta;
            v=X'*theta;
            J_L_est= -(  p_ij'*log(PHI_L(v, sig_L)) + (1-p_ij)'*log(PHI_L(-v, sig_L)) )/M
            %PHI_L( v, sig_L)'
            break;
        end
    end
    %%%% Hessian
    dS= theta - theta';
    PHI_val=PHI_L(dS, sig_G);
    g_fun = PHI_L_prim(dS, sig_G)./PHI_val;
    h_func = PHI_L_bis(dS, sig_G)./PHI_val - g_fun.^2;
    B = C.*h_func;
    H = ( B + B') - diag(sum(B+B',1),0);  %% Hessian
    %%%%%%%
    [Q,LL]=eig(H); diag(LL)';
end

%%% Elo fit
if 1
    sig_L = 600;
    
    theta = 1000+zeros(M,1)*0; %sig_G/sig_model;
    mu=40;
    it=1;
    J_Elo = zeros(It_max,1);
    
    while 1
        % new
        v= X'*theta;
        J_Elo(it)=  -(  p_ij'*log(PHI_G(v, sig_G)) + (1-p_ij)'*log(PHI_G(-v, sig_G)) )/M;
        grad = -X*( ( p_ij - PHI_G(v, sig_G) ) ) ;
        d = grad - (sum(grad))/M;
        theta = theta - mu*d;
        
        %%%%
        it = it+1;
        if it>It_max %|| norm(grad)<ee
            theta_Elo = theta;
            v=X'*theta;
            J_Elo_est = -(  p_ij'*log(PHI_G(v, sig_G)) + (1-p_ij)'*log(PHI_G(-v, sig_G)) )/M
            %PHI_G( v, sig_G )'
            break;
        end
    end
end
plot([J_L J_Elo])
